#version 110

const float maxDistance = sqrt(4.0 * 4.0 + 4.0 * 4.0);

uniform sampler2D tex;
uniform float pixelWidth, pixelHeight, intensity;


void main() {
	vec2 texcoord = gl_TexCoord[0].xy;

	vec4 color = texture2D(tex, texcoord);
	float avg = (color.r + color.g + color.b + color.a) * 0.25;

	vec4 sum = vec4(0.0);
	float highestNeighAvg = 0.0;
	for (float i = -5.0; i <= 5.0; i++) {
		for (float j = -5.0; j <= 5.0; j++) {
			if (i != 0.0 && j != 0.0 && abs(i*j) < 20.0) {
				vec4 neigh = texture2D(tex, texcoord + vec2(i * pixelWidth, j * pixelHeight));
				float neighAvg = (neigh.r + neigh.g + neigh.b + neigh.a) * 0.25;
				float distance = sqrt(i * i + j * j);
				neighAvg -= 0.5 * ((distance / maxDistance));
				if (neighAvg > highestNeighAvg) {
					highestNeighAvg = neighAvg;
				}
			}
		}
	}
	if (highestNeighAvg > avg) {
		float factor = 1.0 / highestNeighAvg * avg;
		color += ((1.0 - factor)/factor) * intensity * color;
	}
	gl_FragColor = color;
}